"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.WinixPurifierAccessory = void 0;
const winix_1 = require("./winix");
const winix = new winix_1.WinixAPI();
class WinixPurifierAccessory {
    constructor(log, config, api) {
        var _a, _b;
        this.hap = api.hap;
        this.log = log;
        const deviceName = config.name;
        this.deviceId = config.deviceId;
        this.latestStatus = {
            power: winix_1.Power.Off,
            mode: winix_1.Mode.Auto,
            airflow: winix_1.Airflow.Low,
            airQuality: winix_1.AirQuality.Good,
            plasmawave: winix_1.Plasmawave.On,
        };
        this.services = [];
        // Create services
        this.purifier = this.registerService(new this.hap.Service.AirPurifier(deviceName));
        const purifierInfo = this.registerService(new this.hap.Service.AccessoryInformation());
        if (config.exposeAirQuality) {
            this.airQuality = this.registerService(new this.hap.Service.AirQualitySensor(`${deviceName} Air Quality`));
        }
        if (config.exposePlasmawave) {
            this.plasmawave = this.registerService(new this.hap.Service.Switch(`${deviceName} Plasmawave`));
        }
        // Assign characteristics
        this.purifier.getCharacteristic(this.hap.Characteristic.Active)
            .onGet(this.getActiveState.bind(this))
            .onSet(this.setActiveState.bind(this));
        this.purifier.getCharacteristic(this.hap.Characteristic.CurrentAirPurifierState)
            .onGet(this.getCurrentState.bind(this));
        this.purifier.getCharacteristic(this.hap.Characteristic.TargetAirPurifierState)
            .onGet(this.getTargetState.bind(this))
            .onSet(this.setTargetState.bind(this));
        this.purifier.getCharacteristic(this.hap.Characteristic.RotationSpeed)
            .onGet(this.getRotationSpeed.bind(this))
            .onSet(this.setRotationSpeed.bind(this));
        purifierInfo
            .setCharacteristic(this.hap.Characteristic.Manufacturer, 'Winix')
            .setCharacteristic(this.hap.Characteristic.Model, config.model);
        (_a = this.airQuality) === null || _a === void 0 ? void 0 : _a.getCharacteristic(this.hap.Characteristic.AirQuality).onGet(this.getAirQuality.bind(this));
        (_b = this.plasmawave) === null || _b === void 0 ? void 0 : _b.getCharacteristic(this.hap.Characteristic.On).onGet(this.getPlasmawave.bind(this)).onSet(this.setPlasmawave.bind(this));
    }
    async getActiveState() {
        const power = await winix.getPower(this.deviceId);
        this.latestStatus.power = power;
        this.log.debug('getActiveState()', power);
        return power === winix_1.Power.On ? this.hap.Characteristic.Active.ACTIVE : this.hap.Characteristic.Active.INACTIVE;
    }
    async setActiveState(state) {
        const power = state === this.hap.Characteristic.Active.ACTIVE ? winix_1.Power.On : winix_1.Power.Off;
        this.log.debug(`setActiveState(${state})`, power);
        if (this.latestStatus.power === power) {
            this.log.debug('ignoring active state set: latestStatus.power === power');
            return;
        }
        await winix.setPower(this.deviceId, power);
        this.latestStatus.power = power;
        this.sendHomekitUpdate();
    }
    async getCurrentState() {
        const power = await winix.getPower(this.deviceId);
        this.latestStatus.power = power;
        this.log.debug('getCurrentState()', power);
        return power === winix_1.Power.On ? this.hap.Characteristic.CurrentAirPurifierState.PURIFYING_AIR :
            this.hap.Characteristic.CurrentAirPurifierState.INACTIVE;
    }
    async getTargetState() {
        const mode = await winix.getMode(this.deviceId);
        this.latestStatus.mode = mode;
        this.log.debug('getTargetState()', mode);
        return mode === winix_1.Mode.Auto ? this.hap.Characteristic.TargetAirPurifierState.AUTO :
            this.hap.Characteristic.TargetAirPurifierState.MANUAL;
    }
    async setTargetState(state) {
        const mode = state === this.hap.Characteristic.TargetAirPurifierState.AUTO ? winix_1.Mode.Auto : winix_1.Mode.Manual;
        this.log.debug(`setTargetState(${state})`, mode);
        // Don't try to set the mode if we're already in this mode
        // Fixes issues with this being set right around the time of power on
        if (this.latestStatus.mode === mode) {
            this.log.debug('ignoring target state set: latestStatus.mode === mode');
            return;
        }
        await winix.setMode(this.deviceId, mode);
        this.latestStatus.mode = mode;
        this.sendHomekitUpdate();
        if (mode === winix_1.Mode.Manual) {
            return;
        }
        // If we're switching back to auto, the airflow speed will most likely change on the Winix device itself.
        // Pause, get the latest airflow speed, then send the update to Homekit
        this.log.debug('scheduling homekit update to rotation speed');
        setTimeout(async () => {
            await this.getRotationSpeed();
            this.sendHomekitUpdate();
        }, 2000);
    }
    async getRotationSpeed() {
        const airflow = await winix.getAirflow(this.deviceId);
        this.latestStatus.airflow = airflow;
        this.log.debug('getRotationSpeed()', airflow);
        return this.toRotationSpeed(airflow);
    }
    async setRotationSpeed(state) {
        const airflow = this.toAirflow(state);
        this.log.debug(`setRotationSpeed(${state})`, airflow);
        await winix.setAirflow(this.deviceId, airflow);
        this.latestStatus.airflow = airflow;
        this.latestStatus.mode = winix_1.Mode.Manual;
        this.sendHomekitUpdate();
    }
    async getAirQuality() {
        const airQuality = await winix.getAirQuality(this.deviceId);
        this.latestStatus.airQuality = airQuality;
        this.log.debug('getAirQuality()', airQuality);
        return this.toAirQuality(airQuality);
    }
    async getPlasmawave() {
        const plasmawave = await winix.getPlasmawave(this.deviceId);
        this.latestStatus.plasmawave = plasmawave;
        this.log.debug('getPlasmawave()', plasmawave);
        return this.toSwitch(plasmawave);
    }
    async setPlasmawave(state) {
        const plasmawave = this.toPlasmawave(state);
        this.log.debug(`setPlasmawave(${state})`, plasmawave);
        await winix.setPlasmawave(this.deviceId, plasmawave);
        this.latestStatus.plasmawave = plasmawave;
        this.sendHomekitUpdate();
    }
    sendHomekitUpdate() {
        var _a, _b;
        this.log.debug('sendHomekitUpdate()');
        this.purifier.updateCharacteristic(this.hap.Characteristic.Active, this.toActiveState(this.latestStatus.power));
        this.purifier.updateCharacteristic(this.hap.Characteristic.CurrentAirPurifierState, this.toCurrentState(this.latestStatus.power));
        this.purifier.updateCharacteristic(this.hap.Characteristic.TargetAirPurifierState, this.toTargetState(this.latestStatus.mode));
        this.purifier.updateCharacteristic(this.hap.Characteristic.RotationSpeed, this.toRotationSpeed(this.latestStatus.airflow));
        (_a = this.airQuality) === null || _a === void 0 ? void 0 : _a.updateCharacteristic(this.hap.Characteristic.AirQuality, this.toAirQuality(this.latestStatus.airQuality));
        (_b = this.plasmawave) === null || _b === void 0 ? void 0 : _b.updateCharacteristic(this.hap.Characteristic.On, this.toSwitch(this.latestStatus.plasmawave));
    }
    toActiveState(power) {
        switch (power) {
            case winix_1.Power.On:
                return this.hap.Characteristic.Active.ACTIVE;
            case winix_1.Power.Off:
                return this.hap.Characteristic.Active.INACTIVE;
        }
    }
    toCurrentState(power) {
        switch (power) {
            case winix_1.Power.On:
                return this.hap.Characteristic.CurrentAirPurifierState.PURIFYING_AIR;
            case winix_1.Power.Off:
                return this.hap.Characteristic.CurrentAirPurifierState.INACTIVE;
        }
    }
    toTargetState(mode) {
        switch (mode) {
            case winix_1.Mode.Auto:
                return this.hap.Characteristic.TargetAirPurifierState.AUTO;
            case winix_1.Mode.Manual:
                return this.hap.Characteristic.TargetAirPurifierState.MANUAL;
        }
    }
    toRotationSpeed(airflow) {
        switch (airflow) {
            case winix_1.Airflow.Sleep:
                return 0;
            case winix_1.Airflow.Low:
                return 25;
            case winix_1.Airflow.Medium:
                return 50;
            case winix_1.Airflow.High:
                return 75;
            case winix_1.Airflow.Turbo:
                return 100;
        }
    }
    toAirflow(state) {
        // Round to nearest 25
        const nearestState = Math.round(state / 25) * 25;
        switch (nearestState) {
            case 0:
                return winix_1.Airflow.Sleep;
            case 25:
                return winix_1.Airflow.Low;
            case 50:
                return winix_1.Airflow.Medium;
            case 75:
                return winix_1.Airflow.High;
            case 100:
                return winix_1.Airflow.Turbo;
        }
        return nearestState > 100 ? winix_1.Airflow.Turbo : winix_1.Airflow.Sleep;
    }
    toAirQuality(airQuality) {
        switch (airQuality) {
            case winix_1.AirQuality.Good:
                return this.hap.Characteristic.AirQuality.GOOD;
            case winix_1.AirQuality.Fair:
                return this.hap.Characteristic.AirQuality.FAIR;
            case winix_1.AirQuality.Poor:
                return this.hap.Characteristic.AirQuality.POOR;
            default:
                return this.hap.Characteristic.AirQuality.UNKNOWN;
        }
    }
    toSwitch(plasmawave) {
        return plasmawave === winix_1.Plasmawave.On;
    }
    toPlasmawave(state) {
        return state ? winix_1.Plasmawave.On : winix_1.Plasmawave.Off;
    }
    registerService(service) {
        this.services.push(service);
        return service;
    }
    getServices() {
        return this.services;
    }
}
exports.WinixPurifierAccessory = WinixPurifierAccessory;
